<?php
declare(strict_types=1);


namespace Lewisdale\App\Models\Data;

use Doctrine\ORM\Mapping as ORM;
use Ramsey\Uuid\Doctrine\UuidGenerator;
use Ramsey\Uuid\UuidInterface;

#[ORM\Entity]
#[ORM\Table(name: 'feed_filters')]
#[ORM\HasLifecycleCallbacks]
class FeedFilter
{
    public function __construct(
        FilterTarget $target,
        FilterType $filter,
        string $value,
        Feed $feed,
        UuidInterface $id = null,
    )
    {
        if ($id !== null) {
            $this->id = $id;
        }

        $this->target = $target;
        $this->filter = $filter;
        $this->value = $value;
        $this->feed = $feed;
    }

    #[ORM\Id]
    #[ORM\Column(type: "uuid", unique: true)]
    #[ORM\GeneratedValue(strategy: "CUSTOM")]
    #[ORM\CustomIdGenerator(class: UuidGenerator::class)]
    public UuidInterface $id;

    public function getId(): UuidInterface
    {
        return $this->id;
    }

    #[ORM\Column(type: 'string', enumType: FilterTarget::class)]
    public FilterTarget $target;

    #[ORM\Column(type: 'string', enumType: FilterType::class)]
    public FilterType $filter;

    #[ORM\Column(type: 'string')]
    public string $value;

    #[ORM\ManyToOne(targetEntity: Feed::class, cascade: ['persist', 'remove'], inversedBy: 'feedFilters')]
    #[ORM\JoinColumn(name: 'feed_id', referencedColumnName: 'id')]
    public Feed $feed;

    public function to_xpath(): string {
        return match ($this->filter) {
            FilterType::EXACT => "{$this->target->value}='{$this->value}'",
            FilterType::INCLUDE => "{$this->target->value}[not(contains(text(), \"{$this->value}\"))]",
            FilterType::EXCLUDE => "{$this->target->value}[contains(text(), \"{$this->value}\")]",
        };
    }

    public function execute(\SimpleXMLElement $item): bool {
        $value = (string) match ($this->target) {
            FilterTarget::TITLE => $item->title,
            FilterTarget::DESCRIPTION => $item->description,
            FilterTarget::LINK => $item->link,
        };

        return match ($this->filter) {
            FilterType::INCLUDE => str_contains(strtolower($value), strtolower($this->value)),
            FilterType::EXACT => $value === $this->value,
            FilterType::REGEX => preg_match($this->value, $value),
            FilterType::EXCLUDE => !str_contains($value, $this->value),
        };
    }
}