<?php
declare(strict_types=1);

namespace Lewisdale\App\Models\Data;

use Lewisdale\App\Models\Traits\AutoUpdate;
use Lewisdale\App\Models\View;
use Doctrine\ORM\Mapping as ORM;

#[ORM\Entity]
#[ORM\Table(name: 'users')]
#[ORM\UniqueConstraint(name: "email", columns: ["email"])]
#[ORM\HasLifecycleCallbacks]
class User
{
    use AutoUpdate;

    #[ORM\Id]
    #[ORM\Column(type: 'integer')]
    #[ORM\GeneratedValue]
    private ?int $id;
    #[ORM\Column(type: 'string')]
    private string $password;
    #[ORM\Column(type: 'string')]
    private string $email;
    #[ORM\Column(type: 'string')]
    private string $salt;

    function __construct(
        string $password,
        string $email,
        ?string $salt = null,
        ?int $id = null,
    ) {
        $this->password = $password;
        $this->email = $email;
        $this->id = $id;
        $this->salt = $salt ?? bin2hex(random_bytes(32));
    }

    public function hash(string $password): string {
        return password_hash($password . $this->salt, PASSWORD_DEFAULT);
    }

    public function validate(string $password): bool {
        return password_verify($password . $this->salt, $this->password);
    }

    public function updatePassword(string $password): void {
        $this->password = $this->hash($password);
    }

    public function toViewModel() : View\User {
        return new View\User(
            $this->email,
            $this->createdAt->format(DATE_ATOM),
            $this->updatedAt->format(DATE_ATOM),
            $this->id
        );
    }
}