<?php
declare(strict_types=1);


namespace Lewisdale\App\Requests;

class Robots
{
    // Original PHP code by Chirp Internet: www.chirpinternet.eu
    // Adapted to include 404 and Allow directive checking by Eric at LinkUp.com
    // Please acknowledge use of this code by including this header.
    public static function allowed(string $url, string | null $useragent = "Baleen"): bool
    {
        // parse url to retrieve host and path
        $parsed = parse_url($url);

        $agents = [preg_quote('*')];
        if ($useragent) {
            $agents[] = preg_quote($useragent, '/');
        }
        $agents = implode('|', $agents);

        // location of robots.txt file, only pay attention to it if the server says it exists
        if (function_exists('curl_init')) {
            $handle = curl_init("http://{$parsed['host']}/robots.txt");
            curl_setopt($handle, CURLOPT_RETURNTRANSFER, TRUE);
            $response = curl_exec($handle);
            $httpCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);
            if (200 == $httpCode) {
                $robots_txt = explode("\n", $response);
            } else {
                $robots_txt = FALSE;
            }
            curl_close($handle);
        } else {
            $robots_txt = @file("http://{$parsed['host']}/robots.txt");
        }

        // if there isn't a robots, then we're allowed in
        if (empty($robots_txt)) {
            return true;
        }

        $rules = [];
        $rule_applies = FALSE;

        foreach ($robots_txt as $line) {
            // skip blank lines
            if (!$line = trim($line)) continue;

            // following rules only apply if User-agent matches $useragent or '*'
            if (preg_match('/^\s*User-agent: (.*)/i', $line, $match)) {
                $rule_applies = preg_match("/($agents)/i", $match[1]);
                continue;
            }

            if ($rule_applies) {
                list($type, $rule) = explode(':', $line, 2);
                $type = trim(strtolower($type));
                // add rules that apply to array for testing
                $rules[] = [
                    'type' => $type,
                    'match' => preg_quote(trim($rule), '/'),
                ];
            }

        }

        $isAllowed = TRUE;
        $currentStrength = 0;

        foreach ($rules as $rule) {
            // check if page hits on a rule
            if (preg_match("/^{$rule['match']}/", $parsed['path'])) {
                // prefer longer (more specific) rules and Allow trumps Disallow if rules same length
                $strength = strlen($rule['match']);
                if ($currentStrength < $strength) {
                    $currentStrength = $strength;
                    $isAllowed = ("allow" == $rule['type']);
                } elseif ($currentStrength == $strength && ("allow" == $rule['type'])) {
                    $currentStrength = $strength;
                    $isAllowed = TRUE;
                }
            }
        }

        return $isAllowed;
    }
}