<?php
declare(strict_types=1);


namespace Lewisdale\App\Tools\Console;

use Symfony\Component\Console\{Attribute\AsCommand,
    Command\Command,
    Input\InputArgument,
    Input\InputInterface,
    Input\InputOption,
    Output\OutputInterface};
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManager;
use Lewisdale\App\Models\Data\Feed;
use Lewisdale\App\Models\Data\FeedFilter;
use Lewisdale\App\Models\Data\FilterTarget;
use Lewisdale\App\Models\Data\FilterType;
use Lewisdale\App\Models\Repositories\FeedRepository;
use function DI\add;

#[AsCommand(name: "test:feed")]
class TestFeed extends Command
{
    public function __construct(private readonly EntityManager $em,
                                private readonly FeedRepository $feedRepository)
    {
        parent::__construct();
    }

    protected function configure(): void
    {
        $optional_array = InputOption::VALUE_IS_ARRAY|InputOption::VALUE_OPTIONAL;
        $this
            ->addArgument("url", InputArgument::REQUIRED, "The URL of the feed to test")
            ->addArgument("title", InputArgument::REQUIRED, "The title of the feed to test")
            ->addOption("filter", "f", $optional_array, "Filter Types to apply to the feed", [])
            ->addOption("target", "t", $optional_array, "Targets to apply the filters to", [])
            ->addOption("value", "a", $optional_array, "Values to apply the filters to", []);
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $output->writeln([
            "Testing Feed",
            "============",
        ]);

        $url = $input->getArgument("url");
        $title = $input->getArgument("title");

        $output->writeln("Setting up a new feed for: $url");

        $feed = new Feed();
        $feed->url = $url;
        $feed->title = $title;

        $filters = array_map(null, $input->getOption("target"), $input->getOption("filter"), $input->getOption("value"));

        $feed->feedFilters = new ArrayCollection();

        foreach ($filters as $filter) {
            $feed->feedFilters->add(new FeedFilter(FilterTarget::from($filter[0]), FilterType::from($filter[1]), $filter[2], $feed));
        }

        $this->em->persist($feed);
        $this->em->flush();

        return Command::SUCCESS;
    }
}