<?php declare(strict_types=1);

use DI\Container;
use Doctrine\DBAL\DriverManager;
use Doctrine\DBAL\Types\Type;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\ORMSetup;
use Lewisdale\App\Logging\FileLogger;
use Lewisdale\App\Models\Types\XMLElement;
use Lewisdale\App\TwigExtensions\CsrfExtension;
use Psr\Log\LoggerInterface;
use Ramsey\Uuid\Doctrine\UuidType;
use Slim\Factory\AppFactory;
use Slim\Views\Twig;

require_once __DIR__ . "/../vendor/autoload.php";

$container = new Container();

AppFactory::setContainer($container);
$app = AppFactory::create();


$container->set('csrf', function() use ($app) {
    return new Slim\Csrf\Guard($app->getResponseFactory());
});

$container->set('view', function() use ($container) {
    $twig = Twig::create(__DIR__ . "/../views");
    $twig->addExtension(new CsrfExtension($container->get('csrf')));

    return $twig;
});

$container->set(Twig::class, function() use ($container) {
    return $container->get('view');
});

$container->set(LoggerInterface::class, $container->get(FileLogger::class));


$container->set(EntityManager::class, static function() {
    Type::addType('uuid', UuidType::class);
    Type::addType(XMLElement::NAME, XMLElement::class);

    $config = ORMSetup::createAttributeMetadataConfiguration(
        paths: array(__DIR__."/Models/Data"),
        isDevMode: true,
    );

    $connection = DriverManager::getConnection([
        'driver' => 'pdo_sqlite',
        'path' => __DIR__ . '/../' . $_ENV["SQLITE_DB_NAME"],
    ], $config);

    $connection->executeQuery("PRAGMA foreign_keys = ON");

    return new EntityManager($connection, $config);
});

